// -*-c++-*-
/* $Id: sfskey.h,v 1.19 1999/04/25 04:44:17 kaminsky Exp $ */

/*
 *
 * Copyright (C) 1999 Michael Kaminsky (kaminsky@lcs.mit.edu)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#ifndef _SFSKEY_H_
#define _SFSKEY_H_ 1

#include "agentmisc.h"
#include "aios.h"
#include "arpc.h"
#include "crypt.h"
#include "sfsmisc.h"
#include "rabin.h"
#include "password.h"
#include "rxx.h"
#include "parseopt.h"

typedef callback<void, bool>::ref cbb;

void addkey (ptr<aclnt> c, str keyfile, cbb, bool imported = false);
void removekey (ptr<aclnt> c, str keyfile, cbb);
void dolist (ptr<aclnt> c, cbb);
void removeallkeys (ptr<aclnt> c, cbb);
void kill_agent (cbb);
void makesymlink (ptr<aclnt> c, str name, str contents, cbb);
void agentc_init ();
bool parse_userhost (const str &raw, str *user, str *host);
void sfs_gethostid (str hostname, sfs_service service,
		    callback<void, str>::ref cb);

class keygen {
  typedef callback<void, int>::ref cb_t;

  str username;
  ptr<rabin_priv> key;
  int nevents;
  str pwd;
  str comment;

  const str keyfile;
  const int bits;
  const bool opt_nopwd;
  const bool opt_kbdnoise;
  const bool opt_b;
  const cb_t cb;

  void finish (int err);

  void decnevents ();
  void savekey ();
  void callkbdnoise ();
  void gotpwd (bool, str);
  void newkey ();
  void reencrypt (ptr<rabin_priv>);
  void editkey (str);
public:
  keygen (str kf, str cmt, int bits, 
	  bool nopwd, bool kbdnoise, bool opt_b, cb_t);
};

class keyregister {
  enum { SK_SRP_EKSBF_COST = 5 };
  typedef callback<void, int>::ref cb_t;

  str username;
  str unixpwd;
  str pwd;
  str key_asc;
  ptr<rabin_priv> key;
  ptr<axprt_stream> authxprt;
  ptr<aclnt> authclnt;
  int fd;

  const str keyfile;
  const str srpfile;
  const bool opt_sec;
  const cb_t cb;

  void finish (int err);

  void doregister_cb (sfsauth_registerres *, clnt_stat);
  void gotsrpparams (sfsauth_srpinfo *, clnt_stat);
  void connected (int fd);
  void gotpwdunix (str);
  void gotpwdkey (str);
public:
  keyregister (str keyfile, str srpfile, bool opt_sec, cb_t);
};

class keyupdate {
  enum { SK_SRP_EKSBF_COST = 5, pubkeybits = 1024 };
  typedef callback<void, int>::ref cb_t;

  ptr<rabin_priv> client_key;

  str newkey_asc;
  str newpwd;
  ptr<rabin_priv> oldkey;
  ptr<rabin_priv> newkey;
  ptr<axprt_stream> authxprt;
  ptr<aclnt> authclnt;
  ptr<aclnt> sfsclnt;

  const str oldkeyfile;
  const str newkeyfile;
  const str srpfile;
  const bool opt_sec;
  str authserv;
  const cb_t cb;

  sfs_hash authhostid;
  str authhostname;

  void finish (int err);
  void keyupdate::client_keygen ();

  void doupdate_cb (sfsauth_updateres *, clnt_stat);
  void gotsrpparams (sfsauth_srpinfo *, sfs_hash, clnt_stat);
  void cryptres (const sfs_hash *);
  void sfsconres (sfs_connectres *, clnt_stat);
  void connected (int fd);
  void connect ();
  void gothostid (str path);
  void gotpwdkey2 (str pwd);
  void gotpwdkey1 (ptr<rabin_priv>);
public:
  keyupdate (str oldkf, str newkf, str srpf, bool opt_sec, str as, cb_t);
};

class keyfetch {
  typedef callback<void, int>::ref cb_t;

  ptr<rabin_priv> key;
  ptr<aclnt> authclnt;
  ptr<axprt_crypt> authxprt;
  str pwd;
  int fd;
  srp srpclnt;
  eksblowfish eksb;
  bool ignorehostid;

  const str keyfile;
  const str authserv;
  const str username;
  const cb_t cb;

  void finish (int err);
  void bfinish (bool b);

  void dosymlink (str name, str contents, bool b);
  void dofetch_cb (sfsauth_fetchres *, clnt_stat);
  void srp_phase0done (sfsauth_srp0res *, clnt_stat);
  void srp_phase1done (sfsauth_srp1res *, clnt_stat);
  void srp_phase2done (sfsauth_srp2res *, clnt_stat);
  void gotconres (int, sfs_connectres *, clnt_stat);
  void connected (int);
  void gotpwd (str);
public:
  keyfetch (str keyfile, str authserv, str username, cb_t);
};

#endif /* _SFSKEY_H_ */
