/* $Id: sfskeyagent.C,v 1.4 1999/04/21 00:36:42 kaminsky Exp $ */

/*
 *
 * Copyright (C) 1999 Michael Kaminsky (kaminsky@lcs.mit.edu)
 * Copyright (C) 1998 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */


#include "sfskey.h"

static void
addkey_3 (bool *bp, cbb cb, clnt_stat err)
{
  if (err)
    fatal << err << "\n";
  if (!*bp) {
    warn << "agent refused key\n";
    cb (false);
  }
  else
    cb (true);
  delete bp;
}

static void
addkey_2 (ptr<aclnt> c, time_t expire, 
	  str *commentp, cbb cb, ptr <rabin_priv> sk)
{
  if (!sk)
    fatal ("could not decode secret key\n");
  sfs_addkey_arg a;
  a.expire = expire;
  a.p = sk->p;
  a.q = sk->q;
  a.comment = *commentp;
  bool *bp = New bool;
  c->call (AGENTCTL_ADDKEY, &a, bp, wrap (addkey_3, bp, cb));
  // XXX - marshalled p and q do not get scrubbed (but we exit soon)

  delete commentp;
}

void
addkey (ptr<aclnt> c, str keyfile, cbb cb, bool imported)
{
  str sk;

  if (!imported) {
    sk = file2wstr (keyfile);
    if (!sk) {
      fatal ("%s: %m\n", keyfile.cstr ());
    }
  }
  else {
    sk = keyfile;
    keyfile = "";
  }

  str *commentp = New str;
  import_rabin_priv_askpwd (sk, "Passphrase for " << keyfile << ": ",
			    wrap (addkey_2, c, 0, commentp, cb),
			    commentp);
}

static void
removekey_2 (bool *bp, cbb cb, clnt_stat err)
{
  if (err)
    fatal << err << "\n";
  if (!*bp) {
    warn << "agent could not remove key\n";
    cb (false);
  }
  else
    cb (true);
  delete bp;
}

void
removekey (ptr<aclnt> c, str keyfile, cbb cb)
{
  str sk (file2wstr (keyfile));
  if (!sk)
    fatal ("%s: %m\n", keyfile.cstr ());

  ptr<rabin_pub> k = import_rabin_pub (sk);

  if (!k)
    fatal << "could not read key from " << keyfile << "\n";

  sfs_remkey_arg a = k->modulus ();
  bool *bp = New bool;
  c->call (AGENTCTL_REMKEY, &a, bp, wrap (removekey_2, bp, cb));
}

static void
dolist_2 (sfs_keylist *klp, cbb cb, clnt_stat err)
{
  if (err)
    fatal << err << "\n";
  sfs_keylistelm *kep = *klp;
  while (kep) {
    warnx ("%6d  0x", kep->expire).cat (kep->key, 16);
    warnx << " " << kep->comment << "\n";
    kep = kep->next;
  }
  cb (true);
}

void
dolist (ptr<aclnt> c, cbb cb)
{
  sfs_keylist *klp = New sfs_keylist;
  c->call (AGENTCTL_DUMP, NULL, klp, wrap (dolist_2, klp, cb));
}

static void
empty_cb (cbb cb, clnt_stat err)
{
  if (err)
    fatal << err << "\n";
  cb (true);
}

void
removeallkeys (ptr<aclnt> c, cbb cb)
{
  c->call (AGENTCTL_REMALL, NULL, NULL, wrap (empty_cb, cb));
}

void
makesymlink (ptr<aclnt> c, str name, str contents, cbb cb)
{
  sfsagent_symlink_arg arg;
  arg.name = name;
  arg.contents = contents;

  c->call (AGENTCTL_SYMLINK, &arg, NULL, wrap (empty_cb, cb));
}

static void
killed_agent (int *resp, cbb cb, clnt_stat stat)
{
  if (stat)
    fatal ("AGENT_KILL: %s\n", clnt_sperrno (stat));
  if (*resp)
    fatal ("AGENT_KILL: %s\n", strerror (*resp));
  cb (true);
  delete resp;
}

void
kill_agent (cbb cb)
{
  int fd = suidgetfd_required ("agent");
  ptr<aclnt> c = aclnt::alloc (axprt_stream::alloc (fd), agent_prog_1);
  int *resp = New int;
  c->call (AGENT_KILL, NULL, resp, wrap (killed_agent, resp, cb));
}

void
agentc_init ()
{
  if (agentc)
    return;

  static rxx sockfdre ("^-(\\d+)$");

  int fd = -1;
  if (agentsock) {
    if (sockfdre.match (agentsock)) {
      fd = atoi (sockfdre[1]);
      if (!isunixsocket (fd))
	fatal << "fd " << fd << " must be unix domain socket with '-S -"
	      << fd << "'\n";
      /* atexit (usrdir_make);  XXX: Why is this? */
    }
  }
  else {
    usrdir_agentsock ();
  }

  if (fd < 0)
    fd = unixsocket_connect (agentsock);
  if (fd < 0)
    fatal ("%s: %m\n", agentsock.cstr ());
  agentc = aclnt::alloc (axprt_stream::alloc (fd), agentctl_prog_1);
}
