/* $Id: suidconnect.C,v 1.4 1999/03/27 20:25:35 dm Exp $ */

/*
 *
 * Copyright (C) 1998 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#include "arpc.h"
#include "sfsagent.h"
#include "sfsmisc.h"

static int clntfd;
static ptr<aclnt> clntc;
static int res;

static void
getres (clnt_stat stat)
{
  if (stat)
    fatal << "SETUIDPROC_SETUID: " << stat << "\n";
  else if (res)
    fatal << "SETUIDPROC_SETUID: " << strerror (res) << "\n";
  else if (!sendfd (0, clntfd))
    exit (0);
  exit (1);
}

int
main (int argc, char **argv)
{
  setprogname (argv[0]);

  if (argc != 2 || !*argv[1]) {
    warnx << "usage: " << progname << " program\n";
    exit (1);
  }
  for (char *pp = argv[1]; *pp; pp++)
    if (!isalnum (*pp) && *pp != '_')
      fatal ("%s: bad program name\n", argv[1]);
  if (!isunixsocket (0))
    fatal ("stdin must be a unix domain socket\n");

#ifdef MAINTAINER
  if (getenv ("SFS_RUNINPLACE")) {
    setgid (getgid ());
    setuid (getuid ());
  }
#endif /* MAINTAINER */

  sfsconst_init ();
  str sock = strbuf ("%s/%s.sock", sfssockdir.cstr (), argv[1]);
  clntfd = unixsocket_connect (sock);
  if (clntfd < 0)
    fatal << sock << ": " << strerror (errno) << "\n";

  clntc = aclnt::alloc (axprt_unix::alloc (clntfd), setuid_prog_1);
  AUTH *a = authunix_create_realids ();
  clntc->call (SETUIDPROC_SETUID, NULL, &res, wrap (getres), a);

  amain ();
}
