/* $Id: authunixint.C,v 1.4 1999/03/27 20:52:35 dm Exp $ */

/*
 *
 * Copyright (C) 1999 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#include "arpc.h"

#ifdef AUTHUNIX_GID_T
typedef gid_t authunix_gid_t;
#else /* !AUTHUNIX_GID_T */
typedef int32_t authunix_gid_t;
#endif /* !AUTHUNIX_GID_T */

AUTH *
authunixint_create (const char *host, u_int32_t uid, u_int32_t gid,
		    u_int32_t ngroups, const u_int32_t *groups)
{
  char *h = const_cast<char *> (host);
  if (sizeof (gid_t) == 4) {
    authunix_gid_t *gids = const_cast<authunix_gid_t *>
      (reinterpret_cast<const authunix_gid_t *> (groups));
    return authunix_create (h, uid, gid, ngroups, gids);
  }

  authunix_gid_t *gids = New authunix_gid_t[ngroups];
  for (size_t i = 0; i < ngroups; i++)
    gids[i] = groups[i];
  AUTH *ret = authunix_create (h, uid, gid, ngroups, gids);
  delete[] gids;
  return ret;
}


/* On some OS's, authunix_create_default uses the effective rather
 * than the read uid and gid.  This makes authunix_create_default
 * unsuitable for use in setuid or setgid programs.  */
AUTH *
authunix_create_realids ()
{
  u_int32_t uid = getuid (), gid = getgid ();
  GETGROUPS_T groups[NGROUPS_MAX];
  int ngroups = getgroups (NGROUPS_MAX, groups);
  if (ngroups < 0)
    ngroups = 0;

  if (sizeof (GETGROUPS_T) == sizeof (authunix_gid_t))
    return authunix_create (const_cast<char *> (myname ().cstr ()),
			    uid, gid, ngroups,
			    reinterpret_cast<authunix_gid_t *> (groups));

  authunix_gid_t *gids = New authunix_gid_t[ngroups];
  for (int i = 0; i < ngroups; i++)
    gids[i] = groups[i];
  AUTH *ret = authunix_create (const_cast<char *> (myname ().cstr ()),
			       uid, gid, ngroups, gids);
  delete[] gids;
  return ret;
}
