/* $Id: rndseed.C,v 1.4 1999/01/27 09:51:30 dm Exp $ */

/*
 *
 * Copyright (C) 1998 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#include "crypt.h"

prng rnd;
sha1oracle rnd_input (64, 0, 8);

static str seedfilepath;
static bool initialized;
static int nupdates;

EXITFN(saveseed);

static void
saveseed ()
{
  if (seedfilepath)
    random_update ();
}

void
random_update ()
{
  u_char buf[64];
  int fd = -1;
  int n = 0;			// g++ bug

  if (seedfilepath) {
    fd = open (seedfilepath, O_RDWR|O_CREAT, 0600);
    if (fd < 0 && errno == EACCES) {
      unlink (seedfilepath);
      fd = open (seedfilepath, O_RDWR|O_CREAT, 0600);
    }
    if (fd >= 0 && (n = read (fd, buf, sizeof (buf))) >= 0)
      rnd_input.update (buf, n);
    else
      warn << seedfilepath << ": " << strerror (errno) << "\n";
  }
  getclocknoise (&rnd_input);

  rnd.seed_oracle (&rnd_input);

  if (fd >= 0) {
    rnd.getbytes (buf, sizeof (buf));
    if (lseek (fd, 0, SEEK_SET) == -1 || write (fd, buf, 48) < 0)
      warn << seedfilepath << ": " << strerror (errno) << "\n";
    close (fd);
  }

  bzero (buf, sizeof (buf));
  nupdates++;
}

static void
random_stir ()
{
  getsysnoise (&rnd_input, wrap (random_update));
}

static void
random_timer ()
{
  random_stir ();
  timecb (time (NULL) + 3600, wrap (random_timer));
}

void
random_init ()
{
  if (initialized)
    panic ("random_init: already initialized\n");
  initialized = true;
  random_timer ();
  while (!nupdates)
    acheck ();
}

void
random_set_seedfile (str path)
{
  if (path[0] == '~' && path[1] == '/') {
    const char *home = getenv ("HOME");
    if (!home) {
      warn ("$HOME not set in environment\n");
      return;
    }

    strbuf pp;
    pp << home;
    const char *p1 = path + 1;
    const char *p2;
    for (; (p2 = strchr (p1 + 1, '/')); p1 = p2) {
      pp.buf (p1, p2 - p1);
      struct stat sb;
      str ppp = pp;
      if (stat (ppp, &sb) < 0
	  && (errno != ENOENT || mkdir (ppp, 0700) < 0)) {
	warn << ppp << ": " << strerror (errno) << "\n";
	return;
      }
    }
    pp << p1;
    seedfilepath = pp;
  }
  else
    seedfilepath = path;
}

void
random_init_file (str path)
{
  random_set_seedfile (path);
  random_init ();
}
