/* $Id: srp.C,v 1.10 1999/04/22 03:00:46 kaminsky Exp $ */

/*
 *
 * Copyright (C) 1999 Michael Kaminsky (kaminsky@lcs.mit.edu)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

/*
 * Secure Remote Password Protocol
 *   http://srp.stanford.edu/srp/
 *
 */

#include "crypt.h"
#include "prime.h"

void
srp::pickN (u_int nbits)
{
  N = random_srpprime (nbits);
}

void
srp::pickg ()
{
  bigint q = (N - 1) / 2;
  g = 2;

  while (1) {
    if (powm (g, q, N) != 1)
      return;
    g = g + 1;
  }
}

bool
srp::set_params (bigint Narg, bigint garg, bool check)
{
  N = Narg;
  g = garg % N;

  if (!check)
    return true;

  if (g == (N - 1) || powm (g, N >> 1, N) != N - 1)
    return false;
  return srpprime_test (N);
}

void
srp::calc_K_cproof ()
{
  sha1ctx sc;
  str raw;

  /* proof = SHA1(N) xor SHA1(g) */
  raw = N.getraw ();
  sha1_hash (proof, raw, raw.len ());
  raw = g.getraw ();
  sha1_hash (K, raw, raw.len ());
  for (u_int i = 0; i < sizeof (proof); i++)
    proof[i] = proof[i] ^ K[i];

  /* Calc K */
  raw = S.getraw ();
  sha1_hash (K, raw, raw.len ());

  /* Calc proof */
  sc.update (proof, sizeof (proof));

  sha1_hash (proof, username, username.len ());
  sc.update (proof, sizeof (proof));

  sc.update (salt, salt.len ());

  raw = A.getraw ();
  sc.update (raw, raw.len ());

  raw = B.getraw ();
  sc.update (raw, raw.len ());

  sc.update (K, sizeof (K));

  sc.final (proof);
}

void
srp::calc_sproof ()
{
  sha1ctx sc;
  str raw;

  raw = A.getraw ();
  sc.update (raw, raw.len ());

  sc.update (proof, sizeof (proof));

  sc.update (K, sizeof (K));

  sc.final (proof);
}

bool
srp::phase0serv (bigint Varg, str _salt, str _user)
{
  v = Varg;
  salt = _salt;
  username = _user;

  return true;
}

bool
srp::phase1clnt ()
{
  a = random_zn (N);
  A = powm (g, a, N);

  return true;
}

bool
srp::phase1serv (bigint Aarg)
{
  A = Aarg;

  b = random_zn (N);
  B = (powm (g, b, N) + v) % N;
  u = random_bigint (UBITS);

  bigint temp = (A * powm (v, u, N)) % N;
  if (temp <= 1 || temp == N - 1)
    return false;

  S = powm (temp, b, N);

  return true;
}

bool
srp::phase2clnt (bigint xArg, bigint Barg, bigint uarg, str _salt, str _user)
{
  v = calc_verifier (xArg);
  B = Barg;
  u = uarg;
  salt = _salt;
  username = _user;

  if ((B % N) == 0 || u == 0) {
    return false;
  }

  S = powm ((B - v), (u * xArg + a), N);

  calc_K_cproof ();

  return true;
}

bool
srp::phase2serv (hash_t clntproof)
{
  calc_K_cproof ();

  for (u_int i = 0; i < sizeof (proof); i++)
    if (clntproof[i] != proof[i])
      return false;

  calc_sproof ();

  return true;
}

bool
srp::phase3clnt (hash_t servproof)
{
  calc_sproof ();

  for (u_int i = 0; i < sizeof (proof); i++)
    if (servproof[i] != proof[i])
      return false;
  
  return true;
}
