// -*-c++-*-
/* $Id: sfsmisc.h,v 1.21 1999/05/14 08:07:33 dm Exp $ */

/*
 *
 * Copyright (C) 1999 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#ifndef _SFSMISC_H_
#define _SFSMISC_H_ 1

#include "nfs3_prot.h"
#include "sfs_prot.h"
#include "amisc.h"

struct svccb;
struct aclnt;
struct asrv;
struct rabin_priv;
struct axprt_crypt;

/* nfs3_err.C */
extern void nfs3_err (svccb *sbp, nfsstat3 status);
extern void nfs3exp_err (svccb *sbp, nfsstat3 status);
const strbuf &strbuf_cat (const strbuf &sb, nfsstat3 err);

/* sfspath.C */
enum { ascii_hostid_len = (sizeof (sfs_hash) * 8 + 4) / 5 };
bool sfs_ascii2hostid (sfs_hash *hid, const char *p);
bool sfsgethost_label (const char *&p);
bool sfsgethost_dotlabel (const char *&p);
str sfsgethost (const char *&p);
bool sfs_parsepath (const str &path, str *host = NULL,
		    sfs_hash *hostid = NULL);
bool sfs_mkhostid (sfs_hash *id, const sfs_hostinfo &info);
bool sfs_ckhostid (const sfs_hash *id, const sfs_hostinfo &info);

/* sfs_err.C */
const strbuf &strbuf_cat (const strbuf &sb, sfsstat err);

/* sfsconst.C */
extern u_int16_t sfs_port;
extern uid_t sfs_uid;
extern gid_t sfs_gid;
#ifdef MAINTAINER
extern bool runinplace;
#else /* !MAINTAINER */
enum { runinplace = false };
#endif /* !MAINTAINER */
extern const char *sfsroot;
extern str sfsdir;
extern str sfssockdir;
extern str sfsdevdb;
extern const char *etc1dir;
extern const char *etc2dir;
extern const char *etc3dir;
void sfsconst_init ();
str sfsconst_etcfile (const char *name);
str sfsconst_etcfile_required (const char *name);

/* sfsaid.C */
typedef u_int64_t sfs_aid;
extern const sfs_aid sfsaid_sfs;
extern const sfs_aid sfsaid_nobody;
sfs_aid aup2aid (const authunix_parms *aup);

/* suidgetfd.C */
int suidgetfd (str prog);
int suidgetfd_required (str prog);

/* unixserv.C */
struct axprt_stream;
typedef callback<void, ptr<axprt_stream>,
                 const authunix_parms *>::ref suidservcb;
void sfs_unixserv (str sock, cbi cb);
void sfs_suidserv (str prog, suidservcb cb);

/* sfssesskey.C */
inline const sfs_kmsg *
sfs_get_kmsg (const str &s)
{
  return reinterpret_cast<const sfs_kmsg *> (s.cstr ());
}
void sfs_get_sesskey (sfs_hash *ksc, sfs_hash *kcs,
		      const sfs_pubkey &ks, const sfs_kmsg *smsg, 
		      const sfs_pubkey &kc, const sfs_kmsg *cmsg);
void sfs_get_sessid (sfs_hash *sessid, const sfs_hash *ksc,
		     const sfs_hash *kcs);
void sfs_get_authid (sfs_hash *authid, sfs_service service, sfs_hostname name,
		     const sfs_hash *hostid, const sfs_hash *sessid);
void sfs_server_crypt (svccb *sbp, rabin_priv *sk, sfs_hash *sessid);
void sfs_client_crypt (ptr<aclnt> c, ptr<rabin_priv> clntkey,
		       const bigint &srvkey,
		       callback<void, const sfs_hash *>::ref cb);
void sfsauth_get_sesskey (sfs_hash *ksc, sfs_hash *kcs, const bigint &key);
void sfsauth_server_crypt (ptr<asrv> s, const bigint &key);
void sfsauth_client_crypt (ptr<aclnt> c, const bigint &key);
#endif /* _SFSMISC_H_ */
