/* $Id: uvfs_init.c,v 1.4 1999/01/19 20:16:50 kaminsky Exp $ */

/*
 * User level VFS driver for OpenBSD.
 * Copyright 1999 Michael Kaminsky <kaminsky@lcs.mit.edu>.
 * Copyright 1998 David Mazieres <dm@uun.org>.
 *
 * Modification and redistribution in source and binary forms is
 * permitted provided that due credit is given to the author and the
 * OpenBSD project (for instance by leaving this copyright notice
 * intact).
 */

#include "uvfs_kern.h"

#include <sys/lkm.h>

static int
loadfs (int cmd, char *name, struct vfsops *ops)
{
  struct vfsconf *vfsp, **vfspp;

  switch(cmd) {
  case LKM_E_LOAD:
    /* make sure there's no VFS in the table with this name */
    for (vfspp = &vfsconf, vfsp = vfsconf; vfsp; 
	 vfspp = &vfsp->vfc_next, vfsp = vfsp->vfc_next)
      if (!strncmp(vfsp->vfc_name, name, MFSNAMELEN))
	return EEXIST;
		
    /* pick the last available empty slot */
    MALLOC (vfsp, struct vfsconf *, sizeof (struct vfsconf), M_VFS, M_WAITOK);

    /* Add tot he end of the list */
    *vfspp = vfsp;

    /*
     * Set up file system
     */
    vfsp->vfc_vfsops = ops;
    strncpy (vfsp->vfc_name, name, MFSNAMELEN);
    vfsp->vfc_name[MFSNAMELEN-1] = '\0';

    vfsp->vfc_typenum = 0;
    vfsp->vfc_refcount = 0;
    vfsp->vfc_flags = 0; /* XXX - should be configurable */
    vfsp->vfc_mountroot = 0;
    vfsp->vfc_next = NULL;

    maxvfsconf++;

    /* Call init function for this VFS... */
    (*(vfsp->vfc_vfsops->vfs_init))(vfsp);

    return 0;
    break;

  case LKM_E_UNLOAD:
    for (vfspp = &vfsconf, vfsp = vfsconf; vfsp &&
	   strncmp(vfsp->vfc_name, name, MFSNAMELEN) != 0;
	 vfspp = &vfsp->vfc_next, vfsp = vfsp->vfc_next)
      ;

    if (!vfsp)
      return EEXIST;

    if (vfsp->vfc_refcount)
      return EBUSY;
		
    *vfspp = vfsp->vfc_next;
    FREE(vfsp, M_VFS);
    maxvfsconf--;
    break;

  case LKM_E_STAT:
    break;
  }

  return 0;
}

static int
uvfs_loadfs (int cmd)
{
  int error;

  switch (cmd) {
  case LKM_E_LOAD:
    vfs_opv_init_explicit (&uvfs_vnodeop_opv_desc);
    vfs_opv_init_default (&uvfs_vnodeop_opv_desc);
    if ((error = loadfs (cmd, "uvfs", &uvfs_vfsops))) {
      FREE (*(uvfs_vnodeop_opv_desc.opv_desc_vector_p), M_VFS);
      return error;
    }
    break;
  case LKM_E_UNLOAD:
    if ((error = loadfs (cmd, "uvfs", &uvfs_vfsops)))
      return error;
    FREE (*(uvfs_vnodeop_opv_desc.opv_desc_vector_p), M_VFS);
    break;
  case LKM_E_STAT:
    break;
  }
  return 0;
}

static int
uvfs_loaddev (struct lkm_table *lkmtp, int cmd, int ver)
{
  MOD_DEV("uvfs", LM_DT_CHAR, -1, &uvfs_cdevsw);
  DISPATCH (lkmtp, cmd, ver, lkm_nofunc, lkm_nofunc, lkm_nofunc)
}

int
uvfs_mod (struct lkm_table *lkmtp, int cmd, int ver)
{
  int error;

  switch (cmd) {
  case LKM_E_LOAD:
    if ((error = uvfs_loaddev (lkmtp, cmd, ver)))
      return error;
    uvfs_dev_init ();
    if ((error = uvfs_loadfs (cmd))) {
      uvfs_loaddev (lkmtp, LKM_E_UNLOAD, ver);
      return error;
    }
    break;
  case LKM_E_UNLOAD:
    if (uvfs_dev_busy ())
      return EBUSY;
    if ((error = uvfs_loadfs (cmd)))
      return error;
    if ((error = uvfs_loaddev (lkmtp, cmd, ver))) {
      uvfs_loadfs (LKM_E_LOAD);
      return error;
    }
    break;
  case LKM_E_STAT:
    if ((error = uvfs_loadfs (cmd))
	|| (error = uvfs_loaddev (lkmtp, cmd, ver)))
      return error;
    break;
  }
  return 0;
}
